/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http//www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is ART Ontology API.
 *
 * The Initial Developer of the Original Code is University of Roma Tor Vergata.
 * Portions created by University of Roma Tor Vergata are Copyright (C) 2009.
 * All Rights Reserved.
 *
 * The ART Ontology API were developed by the Artificial Intelligence Research Group
 * (ai-nlp.info.uniroma2.it) at the University of Roma Tor Vergata
 * Current information about the ART Ontology API can be obtained at 
 * http//ai-nlp.info.uniroma2.it/software/...
 *
 */

/*
 * Contributor(s): Armando Stellato stellato@info.uniroma2.it
 */
package it.uniroma2.art.owlart.models.impl;

import it.uniroma2.art.owlart.exceptions.ModelAccessException;
import it.uniroma2.art.owlart.exceptions.ModelUpdateException;
import it.uniroma2.art.owlart.exceptions.UnavailableResourceException;
import it.uniroma2.art.owlart.exceptions.UnsupportedQueryLanguageException;
import it.uniroma2.art.owlart.model.ARTNode;
import it.uniroma2.art.owlart.model.ARTResource;
import it.uniroma2.art.owlart.model.ARTStatement;
import it.uniroma2.art.owlart.model.ARTURIResource;
import it.uniroma2.art.owlart.model.NodeFilters;
import it.uniroma2.art.owlart.models.BaseRDFTripleModel;
import it.uniroma2.art.owlart.models.RDFModel;
import it.uniroma2.art.owlart.navigation.ARTNodeIterator;
import it.uniroma2.art.owlart.navigation.ARTResourceIterator;
import it.uniroma2.art.owlart.navigation.ARTStatementIterator;
import it.uniroma2.art.owlart.navigation.ARTURIResourceIterator;
import it.uniroma2.art.owlart.query.BooleanQuery;
import it.uniroma2.art.owlart.query.GraphQuery;
import it.uniroma2.art.owlart.query.MalformedQueryException;
import it.uniroma2.art.owlart.query.Query;
import it.uniroma2.art.owlart.query.QueryLanguage;
import it.uniroma2.art.owlart.query.TupleQuery;
import it.uniroma2.art.owlart.vocabulary.RDF;
import it.uniroma2.art.owlart.vocabulary.RDFS;

public class RDFModelImpl extends BaseRDFModelImpl implements RDFModel {

	public RDFModelImpl(BaseRDFTripleModel baseRep) {
		super(baseRep);
	}

	/****************************
	 * /* ADD/REMOVE METHODS ***
	 ****************************/

	public void addInstance(String uri, ARTResource cls, ARTResource... graphs) throws ModelUpdateException {
		baseRep.addTriple(baseRep.createURIResource(uri), RDF.Res.TYPE, cls, graphs);
	}

	public void addProperty(String propertyURI, ARTURIResource superProperty, ARTResource... graphs)
			throws ModelUpdateException {
		ARTURIResource newProp = baseRep.createURIResource(propertyURI);
		baseRep.addTriple(newProp, RDF.Res.TYPE, RDF.Res.PROPERTY, graphs);
		if (superProperty != null)
			baseRep.addTriple(newProp, RDFS.Res.SUBPROPERTYOF, superProperty, graphs);
	}

	public void addType(ARTResource ind, ARTResource cls, ARTResource... graphs) throws ModelUpdateException {
		baseRep.addTriple(ind, RDF.Res.TYPE, cls, graphs);
	}

	public void removeType(ARTResource ind, ARTResource cls, ARTResource... graphs)
			throws ModelUpdateException {
		baseRep.deleteTriple(ind, RDF.Res.TYPE, cls, graphs);
	}

	/*************************
	 * /* BOOLEAN METHODS ***
	 *************************/

	public boolean hasType(ARTResource ind, ARTResource type, boolean inferred, ARTResource... graphs)
			throws ModelAccessException {
		return baseRep.hasTriple(ind, RDF.Res.TYPE, type, inferred, graphs);
	}

	public boolean isProperty(ARTResource prop, ARTResource... graphs) throws ModelAccessException {
		return baseRep.hasTriple(prop, RDF.Res.TYPE, RDF.Res.PROPERTY, true, graphs);
	}

	/*************************
	 * /* RETRIEVE METHODS ***
	 * 
	 * @throws ModelAccessException
	 * @throws UnavailableResourceException
	 *************************/

	/*
	 * public ARTURIResource retrieveProperty(String uri, ARTResource...graphs) throws ModelAccessException {
	 * ARTStatementIterator statIt = baseRep.listStatements(baseRep.createURIResource(uri), RDF.Res.TYPE,
	 * RDF.Res.PROPERTY, true, graphs); if (statIt.streamOpen()) return
	 * statIt.next().getSubject().asURIResource(); else throw new UnavailableResourceException("resource: " +
	 * uri + " is not present in the repository"); }
	 */
	public ARTURIResource retrieveURIResource(String uri, ARTResource... graphs)
			throws ModelAccessException {
		ARTURIResource toBeFound = baseRep.createURIResource(uri);
		
		//first check as subject
		ARTStatementIterator statIt = baseRep.listStatements(toBeFound, NodeFilters.ANY,
				NodeFilters.ANY, false, graphs);
		if (statIt.streamOpen()) {
			ARTURIResource result = statIt.next().getSubject().asURIResource();
			statIt.close();
			return result;
		}
		
		//second check as predicate
		statIt = baseRep.listStatements(NodeFilters.ANY, toBeFound, 
				NodeFilters.ANY, false, graphs);
		if (statIt.streamOpen()) {
			ARTURIResource result = statIt.next().getPredicate();
			statIt.close();
			return result;
		}
		
		//third check as object
		statIt = baseRep.listStatements(NodeFilters.ANY, NodeFilters.ANY, toBeFound, false, graphs);
		if (statIt.streamOpen()) {
			ARTURIResource result = statIt.next().getObject().asURIResource();
			statIt.close();
			return result;
		}
		
		else statIt.close();
		return null;		
	}

	/*************************
	 * /* LIST/GET METHODS ***
	 *************************/

    public ARTResourceIterator listSubjectsOfPredObjPair(ARTURIResource property, ARTNode object, boolean inferred, ARTResource... contexts) throws ModelAccessException {
    	return new SubjectsOfStatementsIterator( baseRep.listStatements(NodeFilters.ANY, property, object, inferred, contexts) );
    }

    public ARTNodeIterator listValuesOfSubjPredPair(ARTResource subject, ARTURIResource predicate, boolean inferred, ARTResource... contexts) throws ModelAccessException {
        return new ObjectsOfStatementsIterator( baseRep.listStatements(subject, predicate, NodeFilters.ANY, inferred, contexts) ) ;
    }

    public ARTURIResourceIterator listPredicatesOfSubjObjPair(ARTResource subject, ARTNode object, boolean inferred, ARTResource... contexts) throws ModelAccessException {
        return new PredicatesOfStatementsIterator( baseRep.listStatements(subject, NodeFilters.ANY, object, inferred, contexts) ) ;
    }
    
	public ARTURIResourceIterator listProperties(ARTResource... graphs) throws ModelAccessException {
		return new URIResourceIteratorWrappingResourceIterator(new SubjectsOfStatementsIterator(baseRep
				.listStatements(NodeFilters.ANY, RDF.Res.TYPE, RDF.Res.PROPERTY, true, graphs)));
	}
	

	public ARTResourceIterator listTypes(ARTResource res, boolean inferred, ARTResource... graphs)
			throws ModelAccessException {
		return new ResourceIteratorWrappingNodeIterator(new ObjectsOfStatementsIterator(baseRep
				.listStatements(res, RDF.Res.TYPE, NodeFilters.ANY, inferred, graphs)));
	}

	public ARTResourceIterator listInstances(ARTResource res, boolean inferred, ARTResource... graphs)
			throws ModelAccessException {
		return new SubjectsOfStatementsIterator(baseRep.listStatements(NodeFilters.ANY, RDF.Res.TYPE, res,
				inferred, graphs));
	}

	/*************************
	 * /* RENAME METHODS ***
	 * 
	 * @throws ModelUpdateException
	 * @throws ModelAccessException
	 *************************/

	public void renameIndividual(ARTURIResource oldIndividual, String newURI) throws ModelUpdateException {
		ARTURIResource newIndividual = baseRep.createURIResource(newURI);
		ARTStatementIterator statIt;
		try {
			statIt = baseRep.listStatements(oldIndividual, NodeFilters.ANY, NodeFilters.ANY, false);
			while (statIt.streamOpen()) {
				ARTStatement st = statIt.next();
				System.err.println("adding quadruple: " + newIndividual + ":" + st.getPredicate() + ":"
						+ st.getObject() + ":" + st.getNamedGraph());
				addTriple(newIndividual, st.getPredicate(), st.getObject(), st.getNamedGraph());
			}
			statIt.close();

			deleteTriple(oldIndividual, NodeFilters.ANY, NodeFilters.ANY);

			statIt = baseRep.listStatements(NodeFilters.ANY, NodeFilters.ANY, oldIndividual, false);
			while (statIt.streamOpen()) {
				ARTStatement st = statIt.next();
				System.err.println("adding quadruple: " + st.getSubject() + ":" + st.getPredicate() + ":"
						+ newIndividual + ":" + st.getNamedGraph());
				addTriple(st.getSubject(), st.getPredicate(), newIndividual, st.getNamedGraph());
			}
			statIt.close();

			deleteTriple(NodeFilters.ANY, NodeFilters.ANY, oldIndividual);

		} catch (ModelAccessException e) {
			throw new ModelUpdateException(e);
		}
	}

	public void renameProperty(ARTURIResource oldProperty, String newPropertyURI) throws ModelUpdateException {
		ARTURIResource newProperty = baseRep.createURIResource(newPropertyURI);
		ARTStatementIterator statIt;
		try {

			statIt = baseRep.listStatements(oldProperty, NodeFilters.ANY, NodeFilters.ANY, false);
			while (statIt.streamOpen()) {
				ARTStatement st = statIt.next();
				addTriple(newProperty, st.getPredicate(), st.getObject(), st.getNamedGraph());
			}
			statIt.close();

			deleteTriple(oldProperty, NodeFilters.ANY, NodeFilters.ANY);

			statIt = baseRep.listStatements(NodeFilters.ANY, oldProperty, NodeFilters.ANY, false);
			while (statIt.streamOpen()) {
				ARTStatement st = statIt.next();
				addTriple(st.getSubject(), newProperty, st.getObject(), st.getNamedGraph());
			}

			deleteTriple(NodeFilters.ANY, oldProperty, NodeFilters.ANY);

			statIt = baseRep.listStatements(NodeFilters.ANY, NodeFilters.ANY, oldProperty, false);
			while (statIt.streamOpen()) {
				ARTStatement st = statIt.next();
				addTriple(st.getSubject(), st.getPredicate(), newProperty, st.getNamedGraph());
			}
			statIt.close();

			deleteTriple(NodeFilters.ANY, NodeFilters.ANY, oldProperty);

		} catch (ModelAccessException e) {
			throw new ModelUpdateException(e);
		}
	}
	
	
	//EXTENSIONS TO TRIPLE QUERY MODEL

	/* (non-Javadoc)
	 * @see it.uniroma2.art.owlart.models.RDFModel#createBooleanQuery(java.lang.String)
	 */
	public BooleanQuery createBooleanQuery(String query) throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		return baseRep.createBooleanQuery(QueryLanguage.SPARQL, query, baseRep.getBaseURI());
	}

	public BooleanQuery createBooleanQuery(String query, String baseURI) throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		return baseRep.createBooleanQuery(QueryLanguage.SPARQL, query, baseURI);
	}

	public BooleanQuery createBooleanQuery(QueryLanguage ql, String query) throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		return baseRep.createBooleanQuery(ql, query, baseRep.getBaseURI());
	}

	public GraphQuery createGraphQuery(String query) throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		return baseRep.createGraphQuery(QueryLanguage.SPARQL, query, baseRep.getBaseURI());
	}

	public GraphQuery createGraphQuery(String query, String baseURI) throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		return baseRep.createGraphQuery(QueryLanguage.SPARQL, query, baseURI);
	}

	public GraphQuery createGraphQuery(QueryLanguage ql, String query) throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		return baseRep.createGraphQuery(ql, query, baseRep.getBaseURI());
	}

	public Query createQuery(String query) throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		return baseRep.createQuery(QueryLanguage.SPARQL, query, baseRep.getBaseURI());
	}

	public Query createQuery(String query, String baseURI) throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		return baseRep.createQuery(QueryLanguage.SPARQL, query, baseURI);
	}

	public Query createQuery(QueryLanguage ql, String query) throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		return baseRep.createQuery(ql, query, baseRep.getBaseURI());
	}

	public TupleQuery createTupleQuery(String query) throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		return baseRep.createTupleQuery(QueryLanguage.SPARQL, query, baseRep.getBaseURI());
	}

	public TupleQuery createTupleQuery(String query, String baseURI) throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		return baseRep.createTupleQuery(QueryLanguage.SPARQL, query, baseURI);
	}

	public TupleQuery createTupleQuery(QueryLanguage ql, String query) throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		return baseRep.createTupleQuery(ql, query, baseRep.getBaseURI());
	}


}
